#!/usr/bin/env python
# -*- coding: utf-8 -*-

# Blackmore's SpamAssassin Warapper v1.1
# Copyright (C) 2010 Simeon Simeonov

# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.

# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.

# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.

import sys
import syslog
import itertools
import subprocess

PROGRAM = """SpamAssassinWrapper"""
VERSION = """1.1"""

SYSLOG_LEVELS = {
    "EMERG": syslog.LOG_EMERG,
    "ALERT": syslog.LOG_ALERT,
    "CRIT": syslog.LOG_CRIT,
    "ERR": syslog.LOG_ERR,
    "WARNING": syslog.LOG_WARNING,
    "NOTICE": syslog.LOG_NOTICE,
    "INFO": syslog.LOG_INFO,
    "DEBUG": syslog.LOG_DEBUG,
    "NONE": False
    }


def get_help():

    return """%s <-h | -v | --pipe <cmd1> [opts] -e <cmd2> [opts]

  - h (Display a description of the command line options understood by SpamAssassinWarapper)
  - l <level> (Set a syslog-priority level. EMERG, ALERT, CRIT (Default), ERR, WARNING, NOTICE, INFO, DEBUG, NONE (for no logging)) 
  - v (Display the version number of the invoked SpamAssassinWarapper)
  - pipe <cmd1> [opts] (Pipe to the specified "cmd1" and provide optional parameters "opts")
""" % sys.argv[0]


def get_version():
    return "%s %s" % (PROGRAM, VERSION)


def pairwise(iterable):
    """ 
    A recipe from:
    http://docs.python.org/release/2.6.6/library/itertools.html#module-itertools
    """
    a, b = itertools.tee(iterable)
    next(b, None)
    return itertools.izip(a, b)


def main():

    syslog_level = syslog.LOG_CRIT
    pipe_index_list = []

    if len(sys.argv) < 2:
        print >> sys.stderr, 'Missing arguments'
        print get_help()
        sys.exit(-1)

    #keeps track of whether we have passed the first pipe argument
    pipe_part = False 

    for i in range (1, len(sys.argv)):

        if ( ((sys.argv[i] == '-h') or (sys.argv[i] == '--help')) and not pipe_part):
            print get_help()
            sys.exit(0)
        elif ( ((sys.argv[i] == '-v') or (sys.argv[i] == '--version')) and not pipe_part):
            print get_version()
            sys.exit(0)
        elif ( ((sys.argv[i] == '-l') or (sys.argv[i] == '--level')) and not pipe_part):
            try:
                syslog_level = SYSLOG_LEVELS[sys.argv[i+1]]
            except:
                syslog_level = syslog.LOG_CRIT
        elif ((sys.argv[i] == '--pipe') and (i < len(sys.argv))):
            pipe_index_list.append(i)
            pipe_part = True

    # insert the last argument in order to make the code simpler
    if len(pipe_index_list):
        pipe_index_list.append(len(sys.argv))
    else:
        print >> sys.stderr, 'No --pipe arguments found'
        print get_help()
        sys.exit(-1)

    if syslog_level:
        syslog.openlog('SpamAssassinWrapper', 0, syslog.LOG_MAIL)


    pipe_iter = pairwise(pipe_index_list)
    popen_list = []
    p = None
    for index in pipe_iter:
        try:
            if p:
                p = subprocess.Popen(sys.argv[index[0]+1:index[1]], stdin=p.stdout, stdout=subprocess.PIPE)
            else:
                p = subprocess.Popen(sys.argv[index[0]+1:index[1]], stdout=subprocess.PIPE)
                popen_list.append(p)

        except OSError as (errno, strerror):
            if syslog_level:
                syslog.syslog(syslog_level, 'OSError: {%s} - %s' % (sys.argv[index[0]+1:index[1]], strerror))
            print >> sys.stderr, 'OSError: {%s} - %s' % (sys.argv[index[0]+1:index[1]], strerror)
            sys.exit(69)
            # 69 - service unavailable (from sysexits.h)
        
        except ValueError:
            if syslog_level:
                syslog.syslog(syslog_level, 'ValueError')
            print >> sys.stderr, 'ValueError'
            sys.exit(69)

        except:
            if syslog_level:
                syslog.syslog(syslog_level, 'Critical error')
            print >> sys.stderr, 'Critical error'
            sys.exit(69)

    try:
        output = p.communicate()[0]
        print output,
    except:
        if syslog_level:
            syslog.syslog(syslog_level, 'Output error')
        print >> sys.stderr, 'Output error'
        sys.exit(69)

    if syslog_level:
        syslog.closelog()


if __name__ == '__main__':
    main()
